@@ -69,6 +69,7 @@ OLDSTYLE_AUTH = re.compile(r"auth=(.*)", re.I)
 class SMTPException(Exception):
     """Base class for all exceptions raised by this module."""
 
+
 class SMTPServerDisconnected(SMTPException):
     """Not connected to any SMTP server.
 
@@ -77,6 +78,7 @@ class SMTPServerDisconnected(SMTPException):
     connecting it to a server.
     """
 
+
 class SMTPResponseException(SMTPException):
     """Base class for all exceptions that include an SMTP error code.
 
@@ -91,6 +93,7 @@ class SMTPResponseException(SMTPException):
         self.smtp_error = msg
         self.args = (code, msg)
 
+
 class SMTPSenderRefused(SMTPResponseException):
     """Sender address refused.
 
@@ -104,6 +107,7 @@ class SMTPSenderRefused(SMTPResponseException):
         self.sender = sender
         self.args = (code, msg, sender)
 
+
 class SMTPRecipientsRefused(SMTPException):
     """All recipient addresses refused.
 
@@ -120,12 +124,15 @@ class SMTPRecipientsRefused(SMTPException):
 class SMTPDataError(SMTPResponseException):
     """The SMTP server didn't accept the data."""
 
+
 class SMTPConnectError(SMTPResponseException):
     """Error during connection establishment."""
 
+
 class SMTPHeloError(SMTPResponseException):
     """The server refused our HELO reply."""
 
+
 class SMTPAuthenticationError(SMTPResponseException):
     """Authentication error.
 
@@ -153,6 +160,7 @@ def quoteaddr(addr):
     else:
         return "<%s>" % m
 
+
 def _addr_only(addrstring):
     displayname, addr = email.utils.parseaddr(addrstring)
     if (displayname, addr) == ('', ''):
@@ -160,6 +168,7 @@ def _addr_only(addrstring):
         return addrstring
     return addr
 
+
 def quotedata(data):
     """Quote data for email.
 
@@ -180,6 +189,7 @@ else:
 
         It only supports what is needed in smtplib.
         """
+
         def __init__(self, sslobj):
             self.sslobj = sslobj
 
@@ -202,6 +212,7 @@ else:
 
     _have_ssl = True
 
+
 class SMTP:
     """This class manages a connection to an SMTP or ESMTP server.
     SMTP Objects:
@@ -268,7 +279,7 @@ class SMTP:
             # if that can't be calculated, that we should use a domain literal
             # instead (essentially an encoded IP address like [A.B.C.D]).
             fqdn = socket.getfqdn()
-            if '.' in fqdn:
+            if '.' in fqdn and fqdn != '.':  # Changed by Kovid
                 self.local_hostname = fqdn
             else:
                 # We can't find an fqdn hostname, so use a domain literal
@@ -789,6 +800,7 @@ class SMTP:
         self.close()
         return res
 
+
 if _have_ssl:
 
     class SMTP_SSL(SMTP):
@@ -826,6 +838,7 @@ if _have_ssl:
 #
 LMTP_PORT = 2003
 
+
 class LMTP(SMTP):
     """LMTP - Local Mail Transfer Protocol
 
